*
* $Id: box_rmin_rmax.F,v 1.2 1995/08/29 13:28:20 zfiles Exp $
*
* $Log: box_rmin_rmax.F,v $
* Revision 1.2  1995/08/29 13:28:20  zfiles
* Complete new set from Rob/Anders
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : box_rmin_rmax
C     Purpose : Finds the max and min radius, z position and 
C               phi
C     
C
C     Modifications:
C
C
C


      subroutine box_rmin_rmax(ivol,iwaf,rmin,rmax,
     +       phi_min,phi_max,zmin,zmax)

      implicit none

      integer ivol,iwaf
      real rmin,rmax,qrmin,qrmax
      real zmin,zmax,qzmin,qzmax
      real phi_min,phi_max,lphi_min,lphi_max

      real xl1(3),xl2(3),xl3(3),xl4(3)
      real xg1(3),xg2(3),xg3(3),xg4(3)

#include "klmn/klmnseq/wafer_n.inc"
#include "klmn/klmnseq/vol.inc"
#include "klmn/klmnseq/klmn_par.inc"

      phi_min=16.3
      phi_max=-6.3
      rmin=10000.0
      rmax=0.0
      zmin=10000.0
      zmax=-10000.0

c     first do plane at z=dz

      xl1(1)=par_vol(ivol,1)
      xl1(2)=par_vol(ivol,2)
      xl1(3)=par_vol(ivol,3)
      
      xl2(1)=-par_vol(ivol,1)
      xl2(2)=par_vol(ivol,2)
      xl2(3)=par_vol(ivol,3)
      
      xl3(1)=par_vol(ivol,1)
      xl3(2)=-par_vol(ivol,2)
      xl3(3)=par_vol(ivol,3)
      
      xl4(1)=-par_vol(ivol,1)
      xl4(2)=-par_vol(ivol,2)
      xl4(3)=par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)


c     first do plane at z=-dz

      xl1(1)=par_vol(ivol,1)
      xl1(2)=par_vol(ivol,2)
      xl1(3)=-par_vol(ivol,3)
      
      xl2(1)=-par_vol(ivol,1)
      xl2(2)=par_vol(ivol,2)
      xl2(3)=-par_vol(ivol,3)
      
      xl3(1)=par_vol(ivol,1)
      xl3(2)=-par_vol(ivol,2)
      xl3(3)=-par_vol(ivol,3)
      
      xl4(1)=-par_vol(ivol,1)
      xl4(2)=-par_vol(ivol,2)
      xl4(3)=-par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      if (lphi_min-phi_min.lt.-3) then
        lphi_min=lphi_min+klmn_twopi
        lphi_max=lphi_max+klmn_twopi
      elseif (phi_min-lphi_min.lt.-3) then
        phi_min=phi_min+klmn_twopi
        phi_max=phi_max+klmn_twopi
      endif

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)


c     first do plane at y=du

      xl1(1)=par_vol(ivol,1)
      xl1(2)=par_vol(ivol,2)
      xl1(3)=par_vol(ivol,3)
      
      xl2(1)=-par_vol(ivol,1)
      xl2(2)=par_vol(ivol,2)
      xl2(3)=par_vol(ivol,3)
      
      xl3(1)=par_vol(ivol,1)
      xl3(2)=par_vol(ivol,2)
      xl3(3)=-par_vol(ivol,3)
      
      xl4(1)=-par_vol(ivol,1)
      xl4(2)=par_vol(ivol,2)
      xl4(3)=-par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      if (lphi_min-phi_min.lt.-3) then
        lphi_min=lphi_min+klmn_twopi
        lphi_max=lphi_max+klmn_twopi
      elseif (phi_min-lphi_min.lt.-3) then
        phi_min=phi_min+klmn_twopi
        phi_max=phi_max+klmn_twopi
      endif

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)


c     first do plane at y=-dy

      xl1(1)=par_vol(ivol,1)
      xl1(2)=-par_vol(ivol,2)
      xl1(3)=par_vol(ivol,3)
      
      xl2(1)=-par_vol(ivol,1)
      xl2(2)=-par_vol(ivol,2)
      xl2(3)=par_vol(ivol,3)
      
      xl3(1)=par_vol(ivol,1)
      xl3(2)=-par_vol(ivol,2)
      xl3(3)=-par_vol(ivol,3)
      
      xl4(1)=-par_vol(ivol,1)
      xl4(2)=-par_vol(ivol,2)
      xl4(3)=-par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      if (lphi_min-phi_min.lt.-3) then
        lphi_min=lphi_min+klmn_twopi
        lphi_max=lphi_max+klmn_twopi
      elseif (phi_min-lphi_min.lt.-3) then
        phi_min=phi_min+klmn_twopi
        phi_max=phi_max+klmn_twopi
      endif

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)



c     first do plane at y=du

      xl1(1)=par_vol(ivol,1)
      xl1(2)=par_vol(ivol,2)
      xl1(3)=par_vol(ivol,3)
      
      xl2(1)=-par_vol(ivol,1)
      xl2(2)=par_vol(ivol,2)
      xl2(3)=par_vol(ivol,3)
      
      xl3(1)=par_vol(ivol,1)
      xl3(2)=par_vol(ivol,2)
      xl3(3)=-par_vol(ivol,3)
      
      xl4(1)=-par_vol(ivol,1)
      xl4(2)=par_vol(ivol,2)
      xl4(3)=-par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      if (lphi_min-phi_min.lt.-3) then
        lphi_min=lphi_min+klmn_twopi
        lphi_max=lphi_max+klmn_twopi
      elseif (phi_min-lphi_min.lt.-3) then
        phi_min=phi_min+klmn_twopi
        phi_max=phi_max+klmn_twopi
      endif

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)


c     first do plane at x=dx

      xl1(1)=par_vol(ivol,1)
      xl1(2)=par_vol(ivol,2)
      xl1(3)=par_vol(ivol,3)
      
      xl2(1)=par_vol(ivol,1)
      xl2(2)=-par_vol(ivol,2)
      xl2(3)=par_vol(ivol,3)
      
      xl3(1)=par_vol(ivol,1)
      xl3(2)=par_vol(ivol,2)
      xl3(3)=-par_vol(ivol,3)
      
      xl4(1)=par_vol(ivol,1)
      xl4(2)=-par_vol(ivol,2)
      xl4(3)=-par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      if (lphi_min-phi_min.lt.-3) then
        lphi_min=lphi_min+klmn_twopi
        lphi_max=lphi_max+klmn_twopi
      elseif (phi_min-lphi_min.lt.-3) then
        phi_min=phi_min+klmn_twopi
        phi_max=phi_max+klmn_twopi
      endif

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)

c     first do plane at x=-dx

      xl1(1)=-par_vol(ivol,1)
      xl1(2)=par_vol(ivol,2)
      xl1(3)=par_vol(ivol,3)
      
      xl2(1)=-par_vol(ivol,1)
      xl2(2)=-par_vol(ivol,2)
      xl2(3)=par_vol(ivol,3)
      
      xl3(1)=-par_vol(ivol,1)
      xl3(2)=par_vol(ivol,2)
      xl3(3)=-par_vol(ivol,3)
      
      xl4(1)=-par_vol(ivol,1)
      xl4(2)=-par_vol(ivol,2)
      xl4(3)=-par_vol(ivol,3)
      
      call transform(iwaf,xl1,xg1)
      call transform(iwaf,xl2,xg2)
      call transform(iwaf,xl3,xg3)
      call transform(iwaf,xl4,xg4)

      call quad_rmin_rmax(xg1,xg2,xg3,xg4,qrmin,qrmax,
     +      lphi_min,lphi_max,qzmin,qzmax)

      zmin=min(zmin,qzmin)
      zmax=max(zmax,qzmax)

      if (lphi_min-phi_min.lt.-3) then
        lphi_min=lphi_min+klmn_twopi
        lphi_max=lphi_max+klmn_twopi
      elseif (phi_min-lphi_min.lt.-3) then
        phi_min=phi_min+klmn_twopi
        phi_max=phi_max+klmn_twopi
      endif

      phi_min=min(phi_min,lphi_min)
      phi_max=max(phi_max,lphi_max)

      rmin=min(rmin,qrmin)
      rmax=max(rmax,qrmax)

      if (rmax.lt.rmin) then

        print *,'Problem in box_rmin_rmax'

      endif

      if (phi_min.gt.klmn_twopi) then
             phi_min=phi_min-klmn_twopi
             phi_max=phi_max-klmn_twopi
      endif


      if (phi_max-phi_min.gt.2.0) then 
         print *,'problem in box_rmin_rmax'
         print *,'phi_max,phi_min:',phi_max,phi_min
      endif

      return 


      end





