*
* $Id: klmn_fit.F,v 1.7 1996/03/12 14:19:54 zfiles Exp $
*
* $Log: klmn_fit.F,v $
* Revision 1.7  1996/03/12 14:19:54  zfiles
* Updates from Anders Ryd
*
* Revision 1.6  1995/12/14 03:52:32  zfiles
* Comment out klmn_arc
*
* Revision 1.5  1995/11/28 19:14:31  zfiles
* Call klmn_arc to compute arc length to TOF
*
* Revision 1.4  1995/09/13  13:50:12  zfiles
* Mods from Rob. Bug fixes
*
* Revision 1.3  1995/09/05  13:14:32  zfiles
*  Remove diagnostic printout.
*  Return error flag when curvature is too large to fit.
*  Change maximum curvature cut to 14. from 10.
*  Use logical instead of integer for klmnerror_error.
*
* Revision 1.2  1995/08/29  13:28:43  zfiles
* Complete new set from Rob/Anders
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : klmn_fit
C     Purpose : See below
C     
C
C     Modifications:
C     Sept  12-95 kutschke:
C                 stepsum is now averaged over one inward and one
C                 outward pass.  Before it was summed over all passes
C                 of all hypotheses.
C     Nov   27-95 kutschke:
C                 Compute stepsum using klmn_arc.  
C     March 11-96 ryd:
C                 Checking the error and return if error
C                 had occured.
C
C
C    This is the routine that the user should call.
C    Depending on what arguments is specified different 
c    fits are performed. It is understood that the common
c    blocks cleofitcom and klmntrk are filled before klmn_fit 
c    is called.
c

c    part_type=1 is electron and positron
c             =2 is muon
c             =3 is pion
c             =4 is kaon
c             =5 is proton
c             =0 fit will be done for all particle types

c   smooth    =.true. smoothing will be performed and track
c                     parameters will be reported at bot iner most 
c                     and outer most point. If particle type is
c                     0 smoothing will be done for the pion
c                     candidate only.
c              .false. the trackfit is only ran invards.

c   residuls  =.true. the residuls for each measurement is
c                     calculated. This is only done if the
c                     the smooth flag is .true..
c              .false. residuls are not calculated.


c   reshit    =.true. the hit is included in the calculation
c                     of the residuals. 
c              .false. the hit is not included in the calculation
c                      of the residuals





      subroutine klmn_fit(part_type,smooth,residuls,reshit)

      implicit none

      integer part_type,ifit
      logical smooth,residuls,reshit

      logical del_hit,del_hit_babar

#include "klmn/klmnseq/klmnctrl.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/klmntrk.inc"
#include "klmn/klmnseq/cleofitcom.inc"
#include "klmn/klmnseq/klmncm.inc"
#include "klmn/klmnseq/klmnhitdel.inc"
#include "klmn/klmnseq/klmnerror.inc"


      ifit=0

      if (abs(k_start).lt.0.01) then
        k_start=0.01*sign(1.0,k_start)
      endif

      if (abs(k_start).gt.14.0) then
        call vfill(klmnerror_error,5,.true.)
        call vfill(klmnerror_nr,5,1)
        goto 999
      endif 


      call vfill(klmnerror_error,5,.false.)
      call vzero(klmnerror_nr,5)

      klmnhitdel_rphidel=0
      klmnhitdel_rzdel=0

      call stop_init

      nmater_n=0

      klmnctrl_buildmater=.true.
      klmnctrl_chisq=.true.
      klmnctrl_savepar=.true.
      klmnctrl_smooth=.true.
      klmnctrl_res=.true.
      klmnctrl_reshit=reshit


5     if (part_type.eq.1) then

         klmnctrl_part=1        

         if (k_start.lt.0) then
           klmnctrl_q=-1.0
         else
           klmnctrl_q=+1.0
         endif

         klmnctrl_mass=0.000511

         call cleofit_lin

      endif
        

      if (part_type.gt.1) then
        
         klmnctrl_part=part_type
 
         if (part_type.eq.2) klmnctrl_mass=0.10566
         if (part_type.eq.3) klmnctrl_mass=0.13957
         if (part_type.eq.4) klmnctrl_mass=0.49368
         if (part_type.eq.5) klmnctrl_mass=0.93827

         call cleofit_lin

      endif
        
      
      if (part_type.eq.0) then

c       first fit pion

        klmnctrl_part=3 
        klmnctrl_mass=0.13957

        call cleofit_lin

      endif     

c     if error occured give up! But first make sure that all
c     hypothesis has error set!

      if (klmnerror_error(klmnctrl_part)) then
        call vfill(klmnerror_error,5,.true.)
        call vfill(klmnerror_nr,5,klmnerror_nr(klmnctrl_part))
        goto 999
      endif

      klmnctrl_buildmater=.false.
      imater_n=1

      ifit=ifit+1
    
      if (del_hit(ifit))   goto 5

c     now do the real fits after doing hit rejection


      klmnctrl_smooth=smooth
      klmnctrl_res=residuls

      if (part_type.eq.0) then

c       next fit electron/positron

        imater_n=1

        klmnctrl_part=1
        if (k_start.lt.0) then
          klmnctrl_q=-1.0
        else
          klmnctrl_q=+1.0
        endif

        klmnctrl_mass=0.000511
        
        call cleofit_lin

c       next fit the muon        

        imater_n=1

        klmnctrl_part=2
        klmnctrl_mass=0.10566

        call cleofit_lin

c       next fit the kaon        

        imater_n=1

        klmnctrl_part=4
        klmnctrl_mass=0.49368

        call cleofit_lin

c       next fit the proton  

        imater_n=1

        klmnctrl_part=5
        klmnctrl_mass=0.93827

        call cleofit_lin

      endif

c     Compute arclength from PCAO to impact on TOF system.
c     The code is not yet production quality.
c      call klmn_arc

999   continue

      return

      end





