*
* $Id: rad_length.F,v 1.3 1996/04/08 14:45:56 zfiles Exp $
*
* $Log: rad_length.F,v $
* Revision 1.3  1996/04/08 14:45:56  zfiles
* Return error if exiting the mother volume
*
* Revision 1.2  1995/08/29  13:28:57  zfiles
* Complete new set from Rob/Anders
*
*
C
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : rad_length
C     Purpose : This routine traces a straight line and calculates how
C               much material it passes through.
C
C     
C
C     Modifications:
C
C
C

      subroutine rad_length(x1,x2,cham_num_in,cham_num_out,
     +                  l_list,s_list,cham_num_list,nentry)

      implicit none

#include "klmn/klmnseq/chamber_n.inc"
#include "klmn/klmnseq/vol.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/klmnerror.inc"

      integer cham_num_in,cham_num_out,cham_num_list(max_list_n)
      integer nentry

      real x1(3),x2(3),l_list(max_list_n),s_list(max_list_n)


      integer i,current_cham,n_entered,entered_list(max_list_n)
      integer entered_done,prev_cham,j
      real    r1,r2,l0,s,s_sum

      integer cham_num_in_svx,cham_num_out_svx
      integer cham_num_list_svx(max_list_n)
      integer nentry_svx

      real l_list_svx(max_list_n),s_list_svx(max_list_n)

      n_entered=0
      entered_done=0

      l0=sqrt((x1(1)-x2(1))**2+(x1(2)-x2(2))**2+(x1(3)-x2(3))**2)
      r1=sqrt(x1(1)**2+x1(2)**2)
      r2=sqrt(x2(1)**2+x2(2)**2)
      if (r1.gt.r2) call swapr(r1,r2)

      nmater_n=nmater_n+1

      nmed=0
      call vzero(med_num_list,50)

      smax=0.0
      last_vol(nmater_n)=cham_num_in


      if (l0.lt.0.0000001) then
        call add_vol(cham_num_in,1.0)
        goto 113
      endif


      cham_num_list(1)=cham_num_in
      s_list(1)=0.0
      nentry=1

      if (cham_num_in.gt.10000) then
         current_cham=cham_svx_n
      else
         current_cham=cham_num_in
      endif

      prev_cham=0

      call add_vol(current_cham,1.0)

 5    do 10,i=daugther_cham(current_cham),
     +   daugther_cham(current_cham)+n_daugther_cham(current_cham)-1

        if (i.eq.prev_cham) goto 10
        if (r1.lt.rmax_cham(i).and.r2.gt.rmin_cham(i)) then
          call in_cham(i,x1,x2,s_list,cham_num_list,nentry,
     +               n_entered,entered_list,s) 
          if (s.ne.0.0) then
             call add_vol(i,s)
             call add_vol(current_cham,-s)
          endif
        endif

 10   continue


 15   entered_done=entered_done+1

      if (entered_done.gt.n_entered) goto 20

      do 30,i=daugther_cham(entered_list(entered_done)),
     +   daugther_cham(entered_list(entered_done))+
     +   n_daugther_cham(entered_list(entered_done))-1


      if (r1.lt.rmax_cham(entered_list(entered_done)).and.
     +    r2.gt.rmin_cham(entered_list(entered_done)))  then
        call in_cham(i,x1,x2,s_list,cham_num_list,nentry,
     +               n_entered,entered_list,s) 

          if (s.ne.0.0) then
             call add_vol(i,s)
             call add_vol(parent_cham(i),-s)
          endif

      endif


 30   continue

      goto 15

 20   continue

c     see if it exits current volume

      call in_cham(current_cham,x1,x2,s_list,cham_num_list,nentry,
     +               n_entered,entered_list,s) 
      
      if (s.ne.0.0) then 
        if (prev_cham.eq.0) then
          call add_vol(current_cham,s-1.0)
        endif
c       protect agains exiting the mother volume
        if (current_cham.ne.1) then
          call add_vol(parent_cham(current_cham),1.0-s)
        endif
      endif

      if (entered_done.gt.n_entered) goto 100   

      prev_cham=current_cham

      current_cham=parent_cham(current_cham)

c     set error condition if exiting the mother volume
      if (current_cham.eq.0) then
        print *, 'rad_length: out of chamber .... '
        call vfill(klmnerror_error,5,.true.)
        call vfill(klmnerror_nr,5,3)
        goto 999
      endif

      goto 5

 100  continue


      cham_num_in_svx=cham_svx_n

#ifdef BABAR
      if (r1.lt.0.16.and.
     +             r2.gt.0.02) then
#else
      if (r1.lt.rmax_cham(cham_svx_n).and.
     +             r2.gt.rmin_cham(cham_svx_n)) then
#endif


        call rad_length_svx(x1,x2,cham_num_in_svx,
     +          cham_num_out_svx,l_list_svx,s_list_svx,
     +          cham_num_list_svx,nentry_svx)


        do 90,i=2,nentry_svx
          nentry=nentry+1
          s_list(nentry)=s_list_svx(i)
          cham_num_list(nentry)=cham_num_list_svx(i)
 90     continue

      endif

 113  continue

      do 215,i=1,nmed_in_list(nmater_n)
          l_list_med(i,nmater_n)=l0*l_list_med(i,nmater_n)
215   continue

      x_mater_n(1,nmater_n)=x1(1)
      x_mater_n(2,nmater_n)=x1(2)
      x_mater_n(3,nmater_n)=x1(3)

      x2_mater_n(1,nmater_n)=x2(1)
      x2_mater_n(2,nmater_n)=x2(2)
      x2_mater_n(3,nmater_n)=x2(3)

      v_mater_n(1,nmater_n)=x2(1)-x1(1)
      v_mater_n(2,nmater_n)=x2(2)-x1(2)
      v_mater_n(3,nmater_n)=x2(3)-x1(3)

      n_in_list(nmater_n)=nentry

      cham_num_out=last_vol(nmater_n)

999   continue

      return

      end









