*
* $Id: rad_length_svx.F,v 1.3 1995/09/05 13:16:04 zfiles Exp $
*
* $Log: rad_length_svx.F,v $
* Revision 1.3  1995/09/05 13:16:04  zfiles
*  Remove diagnostic printout.
*
* Revision 1.2  1995/08/29  13:28:58  zfiles
* Complete new set from Rob/Anders
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : rad_length_svx
C     Purpose : To trace a straight line through the wafer
C               elements and find how much material is passed through.
C     
C
C     Modifications:
C                 
C     July-11-95    Add escape to not search the whole table
C
C
      subroutine rad_length_svx(x1,x2,waf_num_in,waf_num_out,
     +                  l_list,s_list,waf_num_list,nentry)

      implicit none

#include "klmn/klmnseq/wafer_n.inc"
#include "klmn/klmnseq/vol.inc"
#include "klmn/klmnseq/mater_n.inc"
#include "klmn/klmnseq/table.inc"
#include "klmn/klmnseq/chamber_n.inc"
#include "klmn/klmnseq/klmn_par.inc"

      integer waf_num_in,waf_num_out,waf_num_list(max_list_n)
      integer nentry

      real x1(3),x2(3),l_list(max_list_n),s_list(max_list_n)


      integer i,current_waf,n_entered,entered_list(max_list_n)
      integer entered_done,prev_waf,iphi,itheta,iline,k
      real    r1,r2,x0(3),v(3),phi,ctheta,d1,d2,dmax
      real    x(3),x1_p(3),x2_p(3),s
      real    phi1,phi2,phi_min,phi_max,zmin,zmax

      logical in_waf_test

      n_entered=0
      entered_done=0

      phi1=atan2(x1(1),x1(2))
      phi2=atan2(x2(1),x2(2))

      if (phi1-phi2.lt.-klmn_pi) phi1=phi1+klmn_twopi
      if (phi2-phi1.lt.-klmn_pi) phi2=phi2+klmn_twopi

      phi_min=min(phi1,phi2)
      phi_max=max(phi1,phi2)

      zmin=min(x1(3),x2(3))
      zmax=max(x1(3),x2(3))

c      if (phi_max-phi_min.gt.2.0) then
c         print *,'Potential problem in rad_length_svx:'
c         print *,'x1(1),x1(2),x2(1),x2(2):',
c     +            x1(1),x1(2),x2(1),x2(2)
c         print *,'phi1,phi2,phi_min,phi_max:',
c     +            phi1,phi2,phi_min,phi_max
c      endif
      

      r1=sqrt(x1(1)**2+x1(2)**2)
      r2=sqrt(x2(1)**2+x2(2)**2)
      if (r1.gt.r2) call swapr(r1,r2)

      waf_num_list(1)=waf_num_in
      nentry=1

c     find line

      x(1)=x1(1)
      x(2)=x1(2)
      x(3)=x1(3)

      v(1)=x2(1)-x1(1)
      v(2)=x2(2)-x1(2)
      v(3)=x2(3)-x1(3)

      x0(1)=(x1(1)+x2(1))*0.5
      x0(2)=(x1(2)+x2(2))*0.5
      x0(3)=(x1(3)+x2(3))*0.5

      phi=atan2(x0(1),x0(2))

      if (phi.lt.0.0) phi=phi+klmn_twopi

      iphi=nint((phi/klmn_twopi)*nphi+0.5)

      iphi=max(iphi,1)
      iphi=min(iphi,nphi_ptl)

      iline=iphi


      phi=klmn_twopi*(iphi-0.5)/nphi

      phi=phi-phi_min
      

      if (phi.gt.4) phi=phi-klmn_twopi

c      if (abs(phi).gt.1.0) then
c        print *,'phi:',phi,'problem in rad_length_svx'
c      endif

      dmax=max(abs(phi),abs(phi-(phi_max-phi_min)))

c      if (dmax.gt.1.0) then
c        print *,'dmax:',dmax,'problem in rad_length_svx'
c      endif


      if (dist_waf(ntable_waf,iline).gt.dmax) then

      do 4,k=1,ntable_waf

        if (dist_waf(k,iline).gt.dmax) goto 15

        i=number_waf(k,iline)
       
        if (in_waf_test(i,r1,r2,phi_min,phi_max,zmin,zmax)) then
          call in_waf(i,x1,x2,s_list,
     +      waf_num_list,nentry,n_entered,entered_list,s) 
          if (s.ne.0.0) then
            call add_vol(i+10000,s)
            call add_vol(cham_svx_n,-s)
          endif
        endif

 4      continue

      else

c        print *,'Searching all:',dmax,dist_waf(ntable_waf,iline)
c        print *,'x1:',x1(1),x1(2),x1(3)
c        print *,'x2:',x2(1),x2(2),x2(3)

        do 6,k=1,nwaf

          if (level_waf(k).gt.cham_svx_level) goto 6

          if (in_waf_test(k,r1,r2,phi_min,phi_max,zmin,zmax)) then
            call in_waf(k,x1,x2,s_list,
     +               waf_num_list,nentry,n_entered,entered_list,s) 
            if (s.ne.0.0) then
              call add_vol(k+10000,s)
              call add_vol(cham_svx_n,-s)
            endif
          endif

6       continue

      endif

 15   entered_done=entered_done+1

      if (entered_done.gt.n_entered) goto 20

      do 30,i=daugther_waf(entered_list(entered_done)),
     +   daugther_waf(entered_list(entered_done))+
     +   n_daugther_waf(entered_list(entered_done))-1

      
        call in_waf(i-10000,x1,x2,s_list,waf_num_list,nentry,
     +               n_entered,entered_list,s) 
        if (s.ne.0.0) then
          call add_vol(i,s)
          call add_vol(parent_waf(i-10000),-s)
        endif

 30   continue

      goto 15

 20   continue

      return

      end









