*
* $Id: stereo_dist.F,v 1.4 1996/04/08 14:49:21 zfiles Exp $
*
* $Log: stereo_dist.F,v $
* Revision 1.4  1996/04/08 14:49:21  zfiles
* Catch special case of track parallel to wire.
*
* Revision 1.3  1995/08/29  17:35:20  zfiles
* Compatibility with AIX
*
* Revision 1.2  1995/08/29  13:28:59  zfiles
* Complete new set from Rob/Anders
*
*
C
C     Code created by 
C
C     Robert Kutschke : kutschke@lns598.lns.cornell.edu
C     Anders Ryd      : ryd@charm.physics.ucsb.edu
C
C     Created : Sept 1994 -- June 1995
C     Title   : stereo_dist
C     Purpose : finds the point of closest approach of
C               a stereo wire and a track.
C     
C
C     Modifications:
C     April 3   96  ryd:Protect against a divison by zero that happens
C                   when the track is parallel to the wire. When this
C                   happens (abs(v1v2).gt.0.9999) consider the iteration
C                   to have converged and exit. 
C
C
      real function stereo_dist(k,d0,phi,z,t,x0,y0,cx,cy,psi,v1xv2)

      implicit none

      real k,d0,phi,z,t,x0,y0,cx,cy,psi

      real q,x1(3),x2(3),v1(3),v2(3),inv2k,psi2
      real invcxy,invcsclambda,d0pinv2k,cphid0pinv2k,sphid0pinv2k
      real cqpsi,sqpsi,v1v2,s2,v1xv2(3),dpsi,cphi,sphi,spsi,cpsi
      real x0t,y0t,s1,r,invv1xv2


      integer n

      q=sign(1.0,k)

      psi=0.0

      inv2k=0.5/k
      
      invcxy=1.0/sqrt(cx**2+cy**2+1.0)
      invcsclambda=1.0/sqrt(1.0+t**2)
            
      d0pinv2k=d0+inv2k

      cphi=cos(phi)
      sphi=sin(phi)

      cphid0pinv2k=d0pinv2k*cphi
      sphid0pinv2k=d0pinv2k*sphi

      v1(1)=cx*invcxy
      v1(2)=cy*invcxy
      v1(3)=invcxy

      x1(1)=x0
      x1(2)=y0
      x1(3)=0.0

      x0t=-sin(phi)*d0pinv2k
      y0t=cos(phi)*d0pinv2k

      n=0

 10   continue
      cqpsi=cos(phi+q*psi)
      sqpsi=sin(phi+q*psi)
      v1v2=(cx*cqpsi+cy*sqpsi+t)*invcxy*invcsclambda
      x2(1)=-sphid0pinv2k+sqpsi*inv2k
      x2(2)=cphid0pinv2k-cqpsi*inv2k
      x2(3)=z+t*q*inv2k*psi
      v2(1)=cqpsi*invcsclambda
      v2(2)=sqpsi*invcsclambda
      v2(3)=t*invcsclambda
      if (abs(v1v2).gt.0.9999) goto 20
      s2=((x1(1)-x2(1))*(v1(1)*v1v2-v2(1))+
     +    (x1(2)-x2(2))*(v1(2)*v1v2-v2(2))+
     +    (x1(3)-x2(3))*(v1(3)*v1v2-v2(3)))/(v1v2**2-1.0)
      s1=s2*v1v2-v1(1)*(x1(1)-x2(1))-v1(2)*(x1(2)-x2(2))-
     +                                v1(3)*(x1(3)-x2(3))
      r=sqrt((x0t-(x1(1)+s1*v1(1)))**2+
     +                (y0t-(x1(2)+s1*v1(2)))**2)
c      dpsi=s2*abs(2*k)*invcsclambda
      dpsi=s2*invcsclambda/r
      psi=psi+dpsi
      n=n+1

      if (n.gt.50) then 
         print *,'Too many iterations in stereo_dist'
         print *,k,d0,phi,z,t
         print *,x0,y0,cx,cy
         print *,'WARNING WARNING WARNING WARNING WARNING'
         print *,'Returning in stereo_dist without convergence'
         print *,'WARNING WARNING WARNING WARNING WARNING'
         goto 20
      endif

      if (abs(2*s2**2*inv2k).gt.0.000001) goto 10

20    cqpsi=cos(phi+q*psi)
      sqpsi=sin(phi+q*psi)
      x2(1)=-sphid0pinv2k+sqpsi*inv2k
      x2(2)=cphid0pinv2k-cqpsi*inv2k
      x2(3)=z+t*q*inv2k*psi
      v2(1)=cqpsi*invcsclambda
      v2(2)=sqpsi*invcsclambda
      v2(3)=t*invcsclambda


      v1xv2(1)=v1(2)*v2(3)-v1(3)*v2(2)
      v1xv2(2)=v1(3)*v2(1)-v1(1)*v2(3)
      v1xv2(3)=v1(1)*v2(2)-v1(2)*v2(1)

      invv1xv2=1.0/sqrt(v1xv2(1)**2+v1xv2(2)**2+v1xv2(3)**2)

      v1xv2(1)=v1xv2(1)*invv1xv2
      v1xv2(2)=v1xv2(2)*invv1xv2
      v1xv2(3)=v1xv2(3)*invv1xv2

      stereo_dist=-((x1(1)-x2(1))*v1xv2(1)+
     +              (x1(2)-x2(2))*v1xv2(2)+
     +              (x1(3)-x2(3))*v1xv2(3))

      return

      end






